package com.ims.eval.controller;

import com.baomidou.mybatisplus.core.metadata.IPage;
import com.ims.eval.config.CustomException;
import com.ims.eval.entity.EvaluationDept;
import com.ims.eval.entity.EvaluationDeptBusinessAssessment;
import com.ims.eval.entity.EvaluationDeptBusinessContent;
import com.ims.eval.entity.EvaluationDeptBusinessPlan;
import com.ims.eval.entity.custom.Path;
import com.ims.eval.entity.dto.result.R;
import com.ims.eval.service.IEvaluationDeptBusinessAssessmentService;
import com.ims.eval.service.IEvaluationDeptBusinessContentService;
import com.ims.eval.util.WordUtil;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.web.bind.annotation.*;

import javax.servlet.ServletOutputStream;
import javax.servlet.http.HttpServletRequest;
import javax.servlet.http.HttpServletResponse;
import java.io.*;
import java.net.URLEncoder;
import java.nio.file.Files;
import java.util.*;

/**
 * 本部部门业绩指标考核
 *
 * @author hlf
 * @date 2023/4/28 10:09
 * 文件说明：
 */
@RestController
@RequestMapping("//evaluation-dept-assessment")
public class DepartmentalPerformanceIndicatorAssessmentController {

	@Autowired
	private IEvaluationDeptBusinessAssessmentService evaluationDeptBusinessAssessmentService;

	@Autowired
	private IEvaluationDeptBusinessContentService evaluationDeptBusinessContentService;

	/**
	 * 部门考评业务列表信息（分页）
	 *
	 * @param pageNum            当前记录起始索引
	 * @param pageSize           每页显示记录数
	 * @param responsibilityCode 业务编码
	 * @param des                业务简述
	 * @param appraisalYear      月度
	 * @param stage              流程状态
	 * @return 结果
	 */
	@GetMapping(value = "/list")
	public R list(
		@RequestParam(value = "pageNum") Integer pageNum,
		@RequestParam(value = "pageSize") Integer pageSize,
		@RequestParam(value = "responsibilityCode", required = false) String responsibilityCode,
		@RequestParam(value = "des", required = false) String des,
		@RequestParam(value = "appraisalYear", required = false) String appraisalYear,
		@RequestParam(value = "stage", required = false) String stage) {
		IPage<EvaluationDeptBusinessAssessment> list = evaluationDeptBusinessAssessmentService.listPage(pageNum, pageSize, responsibilityCode, des, appraisalYear, stage);
		return R.ok().data(list);
	}

	/**
	 * 部门考评业务信息
	 *
	 * @return 结果
	 */
	@GetMapping(value = "/listAll")
	public R listAll() {
		List<EvaluationDeptBusinessAssessment> list = evaluationDeptBusinessAssessmentService.list();
		return R.ok().data(list);
	}

	/**
	 * 生成报表
	 *
	 * @return 结果
	 */
	@GetMapping(value = "/generateReport/{id}")
	public R generateReport(@PathVariable("id") String id) throws Exception {
		Map<String, Object> dataMap = new HashMap<>();
		EvaluationDeptBusinessAssessment evaluationDeptBusinessAssessment = evaluationDeptBusinessAssessmentService.getById(id);
		String[] deptIds = evaluationDeptBusinessAssessment.getDeptId().split(",");
		String[] deptNames = evaluationDeptBusinessAssessment.getDeptName().split(",");
		List<EvaluationDept> evaluationDeptList = new ArrayList<>();
		for (int i = 0; i < deptIds.length; i++) {
			EvaluationDept evaluationDept = new EvaluationDept();
			evaluationDept.setDeptId(deptIds[i]);
			evaluationDept.setDeptName(deptNames[i]);
			List<EvaluationDeptBusinessContent> evaluationDeptBusinessContentList = evaluationDeptBusinessContentService.selectEvaluationDeptBusinessContentListByBusinessPlanId(id, deptIds[i]);
			evaluationDept.setList(evaluationDeptBusinessContentList);
			evaluationDeptList.add(evaluationDept);
		}
		dataMap.put("appraisalYear", evaluationDeptBusinessAssessment.getAppraisalYear());
		dataMap.put("list", evaluationDeptList);
		String fileName = evaluationDeptBusinessAssessment.getAppraisalYear() + "年度经营业绩完成情况.doc";
		WordUtil.generateWord(dataMap, "template_dbkp_wcqk.ftl", Path.getStatementPath(), fileName);
		return R.ok().data(fileName);
	}

	/**
	 * 下载报表
	 *
	 * @param request
	 * @param response
	 * @param reportName
	 * @throws IOException
	 */
	@GetMapping(value = "/downloadReport/{reportName}")
	public void downloadReport(HttpServletRequest request, HttpServletResponse response, @PathVariable("reportName") String reportName) throws IOException {
		InputStream inputStream = null;
		ServletOutputStream outputStream = null;
		try {
			File file = new File(Path.getStatementPath() + reportName);
			inputStream = Files.newInputStream(file.toPath());
			response.setCharacterEncoding("UTF-8");
			response.setContentType("application/msword");
			response.addHeader("Access-Control-Allow-Origin", request.getHeader("Origin"));
			response.addHeader("Access-Control-Expose-Headers", "token, uid, Content-Disposition");
			response.addHeader("Access-Control-Allow-Methods", "GET, POST, PUT, DELETE");
			response.addHeader("Access-Control-Allow-Headers", "Content-Type");
			response.addHeader("Access-Control-Allow-Credentials", "true");
			// 设置浏览器以下载的方式处理该文件名
			response.setHeader("Content-Disposition", "attachment;filename=".concat(String.valueOf(URLEncoder.encode(reportName, "UTF-8"))));
			outputStream = response.getOutputStream();
			byte[] buffer = new byte[512];  // 缓冲区
			int bytesToRead = -1;
			// 通过循环将读入的Word文件的内容输出到浏览器中
			while ((bytesToRead = inputStream.read(buffer)) != -1) {
				outputStream.write(buffer, 0, bytesToRead);
			}
		} finally {
			if (inputStream != null) inputStream.close();
			if (outputStream != null) outputStream.close();
		}
	}

	/**
	 * 修改状态
	 *
	 * @param id    主键
	 * @param stage 状态
	 * @return 结果
	 */
	@PostMapping(value = "/targetStart")
	public R targetStart(@RequestParam(value = "id") String id,
						 @RequestParam(value = "stage") String stage) {
		try {
			EvaluationDeptBusinessAssessment evaluationDeptBusinessAssessment = evaluationDeptBusinessAssessmentService.getById(id);
			evaluationDeptBusinessAssessment.setStage(stage);
			boolean b = evaluationDeptBusinessAssessmentService.updateById(evaluationDeptBusinessAssessment);
			if (b) {
				return R.ok().data(b);
			} else {
				return R.error().data("失败！");
			}
		} catch (CustomException e) {
			return R.customError(e.getMessage()).data("失败！");
		}
	}

	/**
	 * 详情-头部信息
	 *
	 * @param id 业务主键
	 * @return 结果
	 */
	@GetMapping(value = "/detailsHead/{id}")
	public R detailsHead(@PathVariable("id") String id) {
		EvaluationDeptBusinessAssessment evaluationDeptBusinessAssessment = evaluationDeptBusinessAssessmentService.getById(id);
		return R.ok().data(evaluationDeptBusinessAssessment);
	}
}
