package com.ims.eval.controller;

import com.baomidou.mybatisplus.core.metadata.IPage;
import com.ims.eval.config.CustomException;
import com.ims.eval.entity.EvaluationDeptRating;
import com.ims.eval.entity.EvaluationNotice;
import com.ims.eval.entity.dto.result.R;
import com.ims.eval.service.IEvaluationNoticeService;
import lombok.extern.slf4j.Slf4j;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.web.bind.annotation.*;
import org.springframework.web.multipart.MultipartFile;

import java.io.IOException;
import java.util.Arrays;
import java.util.List;

/**
 * 通告管理
 *
 * @author hlf
 * @date 2023/5/17 14:48
 * 文件说明：
 */
@Slf4j
@RestController
@RequestMapping("//evaluation-dept-notice")
public class NoticeManagementController {

	@Autowired
	private IEvaluationNoticeService evaluationNoticeService;

	/**
	 * 通告管理列表信息（分页）
	 *
	 * @param pageNum     当前记录起始索引
	 * @param pageSize    每页显示记录数
	 * @param noticeTitle 通告标题
	 * @return 结果
	 */
	@GetMapping(value = "/list")
	public R list(
		@RequestParam(value = "pageNum") Integer pageNum,
		@RequestParam(value = "pageSize") Integer pageSize,
		@RequestParam(value = "noticeTitle", required = false) String noticeTitle) {
		IPage<EvaluationNotice> list = evaluationNoticeService.listPage(pageNum, pageSize, noticeTitle);
		return R.ok().data(list);
	}

	/**
	 * 通告管理信息
	 *
	 * @return 结果
	 */
	@GetMapping(value = "/listAll")
	public R listAll() {
		List<EvaluationNotice> list = evaluationNoticeService.list();
		return R.ok().data(list);
	}

	/**
	 * 新增/修改通告管理信息
	 *
	 * @param evaluationNotice 通告管理实体
	 * @return 结果
	 */
	@PostMapping(value = "/save")
	public R addAll(EvaluationNotice evaluationNotice, @RequestParam(value ="files", required = false) MultipartFile[] files) throws IOException {
		try {
			boolean b = evaluationNoticeService.saveOrUpdate(evaluationNotice, files);
			if (b) {
				return R.ok().data(b);
			} else {
				return R.error().data("保存失败！");
			}
		} catch (CustomException e) {
			return R.customError(e.getMessage()).data("失败！");
		}
	}

	/**
	 * 批量删除通告管理信息
	 *
	 * @param ids
	 * @return 结果
	 */
	@PostMapping(value = "/removeAll/{ids}")
	public R deleteAll(@PathVariable("ids") String ids) {
		String[] strings = ids.split(",");
		boolean b = evaluationNoticeService.removeByIds(Arrays.asList(strings));
		if (b) {
			return R.ok().data(b);
		} else {
			return R.error().data("删除失败！");
		}
	}

	/**
	 * 发布通告
	 *
	 * @param id           主键
	 * @param releaseState 发布状态
	 * @return 结果
	 */
	@PostMapping(value = "/issueNotice")
	public R issueNotice(@RequestParam(value = "id") String id,
						 @RequestParam(value = "releaseState") String releaseState) {
		EvaluationNotice evaluationNotice = new EvaluationNotice();
		evaluationNotice.setId(id);
		evaluationNotice.setReleaseState(releaseState);
		boolean b = evaluationNoticeService.updateById(evaluationNotice);
		if (b) {
			return R.ok().data(b);
		} else {
			return R.error().data("发布失败！");
		}
	}
}
