package com.ims.eval.util;

import com.baomidou.mybatisplus.core.toolkit.CollectionUtils;
import org.apache.commons.lang3.StringUtils;
import org.apache.poi.hssf.usermodel.HSSFCell;
import org.apache.poi.hssf.usermodel.HSSFRow;
import org.apache.poi.hssf.usermodel.HSSFSheet;
import org.apache.poi.hssf.usermodel.HSSFWorkbook;

import java.beans.BeanInfo;
import java.beans.IntrospectionException;
import java.beans.Introspector;
import java.beans.PropertyDescriptor;
import java.lang.reflect.Field;
import java.lang.reflect.InvocationTargetException;
import java.lang.reflect.Method;
import java.text.SimpleDateFormat;
import java.util.HashMap;
import java.util.List;
import java.util.Map;

/**
 * @author ：xugp
 * @date ：Created in 2023/4/14 10:42
 * @description：
 * @modified By：
 * @version: $
 */
public class ExcelUtils {

	/**
	 *  workbook
	 * @param titleList
	 * @return
	 */
	public static HSSFWorkbook getWorkBook(List<String> titleList){
		//第一步，创建一个workbook，对应一个Excel文件
		HSSFWorkbook wb = new HSSFWorkbook();
		// 一个sheet
		HSSFSheet sheet = wb.createSheet("sheet1");

		HSSFRow rowTitle = sheet.createRow(0); // 第一行 标题

		// 设置标题
		for (int i = 0; i < titleList.size(); i++) {
			HSSFCell cell = rowTitle.createCell(i);
			cell.setCellValue(titleList.get(i));
		}
		//合并单元格CellRangeAddress构造参数依次表示起始行，截至行，起始列， 截至列
        /*sheet.addMergedRegion(new CellRangeAddress(0,0,0,4));
        sheet.addMergedRegion(new CellRangeAddress(titleList.size()-1,titleList.size()-1,titleList.size()-1,titleList.size()+1));*/
		return wb;
	}


	public static <T> HSSFWorkbook getWorkBook(List<String> titleList , List<T> dataList) throws IntrospectionException, InvocationTargetException, IllegalAccessException {
		if (CollectionUtils.isNotEmpty(dataList)) {
			T t1 = dataList.get(0);
			Class<?> t1Class = t1.getClass();
			EnableExcel enableExcel = t1Class.getAnnotation(EnableExcel.class);
			if (enableExcel == null) {
				throw new IllegalArgumentException("EnableExcel 注解没有在实体类启用");
			}
			Field[] fields = t1Class.getDeclaredFields();
			if (fields != null && fields.length > 0) {
				Map<String , Integer> titleMap = new HashMap<>(titleList.size()); // 存放属性名称对应的下标

				int fieldExcelSize = 0; // 类中ExcelRow 注解的数量
				for (Field field : fields) {
					field.setAccessible(true);
					String fieldName = field.getName();
					ExcelRow excelRow = field.getAnnotation(ExcelRow.class);
					if (excelRow != null) {
						String name = excelRow.name();
						if (StringUtils.isEmpty(name)) {
							throw new IllegalArgumentException("ExcelRow 注解name属性不能为空");
						}

						int index = titleList.indexOf(name.trim());
						if (index != -1) {
							fieldExcelSize++;
							titleMap.put(fieldName , index);
						}
					}
				}

				if (!(titleList.size() == fieldExcelSize)) {
					throw new IllegalArgumentException("ExcelRow 注解name属性对应的列数不对");
				}

				HSSFWorkbook workBook = getWorkBook(titleList);
				HSSFSheet sheet = workBook.getSheetAt(0);

				for (T t : dataList) {
					int lastRowNum = sheet.getLastRowNum();
					HSSFRow row = sheet.createRow(lastRowNum + 1);
					BeanInfo beanInfo = Introspector.getBeanInfo(t.getClass());
					PropertyDescriptor[] propertyDescriptors = beanInfo.getPropertyDescriptors();
					for (PropertyDescriptor propertyDescriptor : propertyDescriptors) {
						String fieldName = propertyDescriptor.getName();
						if (titleMap.containsKey(fieldName)) {
							Method readMethod = propertyDescriptor.getReadMethod();
							if (readMethod != null) {
								Class<?> returnType = readMethod.getReturnType();
								String simpleName = returnType.getSimpleName();

								Object invoke = readMethod.invoke(t);
								String value = "";
								// 可以根据不同的类型返回不同的数据
								if ("date".equalsIgnoreCase(simpleName)) {
									SimpleDateFormat simpleDateFormat = new SimpleDateFormat("yyyy-MM-dd HH:mm:ss");
									if (invoke != null) {
										value = simpleDateFormat.format(invoke);
									}
								}
								if (invoke != null && "".equals(value)) {
									value = invoke.toString();
								}
								row.createCell(titleMap.get(fieldName)).setCellValue(value);
							}
						}
					}
				}
				return workBook;
			}
		}
		return null;
	}
}


